package com.WebFluxReactiveMongo.Reactor.Controller;

import com.WebFluxReactiveMongo.Reactor.Dto.StudentDto;
import com.WebFluxReactiveMongo.Reactor.Service.StudentService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.support.serializer.JsonSerializer;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderOptions;

import java.util.HashMap;
import java.util.Map;


@RestController
@RequestMapping("/students")
@Slf4j
public class StudentController {

    @Autowired
    private StudentService studentService;

    private static final String TOPIC = "TestTopic1";


    public static KafkaSender sampleProducer() {

        Map<String, Object> props = new HashMap<>();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, "localhost:9092");
        props.put(ProducerConfig.CLIENT_ID_CONFIG, "sample-producer");
        props.put(ProducerConfig.ACKS_CONFIG, "all");
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, JsonSerializer.class);
        SenderOptions<String, JsonNode> senderOptions = SenderOptions.create(props);

        return KafkaSender.create(senderOptions);
    }
    @GetMapping
    public Flux<StudentDto> getStudents(){
        return studentService.getStudents();
    }

    /*@GetMapping
    public Flux<Student> getStudentsTemp(){
        log.info("inside controller=========");
        return studentService.getStudentsWithTemp();
    }*/

    @GetMapping("/{id}")
    public Mono<StudentDto> studentById(@PathVariable String id){
        return studentService.getStudentById(id);
    }

    @GetMapping("/student-fee-range")
    public Flux<StudentDto> getStudentFeeRange(@RequestParam("min") double min, @RequestParam("max") double max){
        return studentService.getStudentInFee(min,max);
    }

    @PostMapping
    public Mono<StudentDto> saveStudent(@RequestBody Mono<StudentDto> studentDtoMono){
        return studentService.saveStudent(studentDtoMono);
    }

    @PutMapping("/update/{id}")
    public Mono<StudentDto> updateStudent(@RequestBody Mono<StudentDto> studentDtoMono, @PathVariable String id){
        return studentService.updateStudent(studentDtoMono,id);
    }

    @DeleteMapping("/{id}")
    public Mono<Void> deleteStudent(@PathVariable String id){
        return studentService.deleteStudent(id);
    }

    @GetMapping("/publishReactorJson")
    public Mono<StudentDto> sendMessages(@RequestBody StudentDto studentDto) throws InterruptedException, JsonProcessingException {
        Mono<StudentDto> stockDTO =studentService.saveStudent(Mono.just(studentDto));

        return stockDTO.flatMap(data -> sendToKafka(data, "create"));

    }

    private Mono<StudentDto> sendToKafka(StudentDto studentDto, String create) {
        return StudentController.sampleProducer().createOutbound()
                .send(Mono.just(new ProducerRecord<>("TestTopic1",
                        "dto.getSalt()", studentDto))).then().log()
                .doOnError(e -> log.error(
                        String.format("Failed to send topic: %s value: %s",
                                "TestTopic1", studentDto), e))
                .thenReturn(studentDto);
    }


}
